/*
 * Copyright (C) 2006, 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "CookieJar.h"

#include "Cookie.h"
#include "KURL.h"
#include "Document.h"
#include "ResourceHandle.h"
#include <windows.h>
#include <Wininet.h>
#include <Iepmapi.h>
#include <wtf/text/WTFString.h>

#include "WebCoreInstanceHandle.h"

namespace WebCore {

void setCookies(Document* /*document*/, const KURL& url, const String& value)
{
#if 0
	// Is IE protected mode ? (These proc may be necessary, but we have no time...)
	BOOL IsIEProtectedMode = FALSE;
    HRESULT hRes = IEIsProtectedModeProcess(&IsIEProtectedMode);
	if (FAILED(hRes)) {
	//	MYDEBUGMSG("*** Error IEIsProtectedModeProcess ***\n");
		IsIEProtectedMode = FALSE;
	} else {
		if (IsIEProtectedMode) {
			MYDEBUGMSG("IsIEProtectedMode = %d\n", IsIEProtectedMode);
		}
	}
	//MYDEBUGMSG("IsIEProtectedMode = %d\n", IsIEProtectedMode);
#endif
    // FIXME: Deal with document->firstPartyForCookies().
    String str = url.string();
    String val = value;

	InternetSetCookieW(str.charactersWithNullTermination(), 0, val.charactersWithNullTermination());
}

String cookies(const Document* /*document*/, const KURL& url)
{
    String str = url.string();

    DWORD count = 4096;
	TCHAR* buffcookie = new TCHAR[4096];
	if (!InternetGetCookieW(str.charactersWithNullTermination(), 0, buffcookie, &count)) {
		DWORD dwError = GetLastError();
		if (dwError != ERROR_INSUFFICIENT_BUFFER || count <= 1) {
			delete[] buffcookie;
        return String();
		} 
		delete[] buffcookie;
		buffcookie = new TCHAR[count];
		if (!InternetGetCookieW(str.charactersWithNullTermination(), 0, buffcookie, &count)) {
			delete[] buffcookie;
        return String();
		}
	}
    String strCookie = String(buffcookie, count);
	delete[] buffcookie;
    return strCookie;
}

String cookieRequestHeaderFieldValue(const Document* document, const KURL& url)
{
    // FIXME: include HttpOnly cookie
    return cookies(document, url);
}

bool cookiesEnabled(const Document* /*document*/)
{
    return true;
}

bool getRawCookies(const Document*, const KURL&, Vector<Cookie>& rawCookies)
{
    // FIXME: Not yet implemented
    rawCookies.clear();
    return false; // return true when implemented
}

void deleteCookie(const Document*, const KURL&, const String&)
{
    // FIXME: Not yet implemented
}

void getHostnamesWithCookies(HashSet<String>& hostnames)
{
    // FIXME: Not yet implemented
}

void deleteCookiesForHostname(const String& hostname)
{
    // FIXME: Not yet implemented
}

void deleteAllCookies()
{
    // FIXME: Not yet implemented
}

}
