/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "WebCoreInstanceHandle.h"

namespace WebCore {

HINSTANCE s_instanceHandle;
volatile int s_ini_EnableDebugPrint = 0;	/* -2:busy, 0:never print, 1:only js print, 2: all print */
int s_ini_MemCheckInterval = 80;
int s_ini_MaxMemoryLoad = 78;
DWORD s_ini_MaxWorkingSet = 1610612736;	// 1.5G
DWORD s_ini_HttpUseProtocolCachePolicy = 0x8440c000;	// No Use Cache Mechanism of WinInet (If u want to use Cache Mechanism, set '0x40c800'
DWORD s_ini_HttpReloadIgnoringCacheData = 0x8440c000;
DWORD s_ini_HttpReturnCacheDataElseLoad = 0x8440c000;
DWORD s_ini_HttpReturnCacheDataDontLoad = 0x8440c000;
int s_ini_HttpDefaultTimeOut = 0;
int s_ini_GesturePanEnable = 2;
int s_ini_TransForm3D = 0;
int s_IsProcessImmersive = 0;
int s_ini_HttpSynchronousMode = 5;	// bit1:use webkitinet.exe, bit2:always disable sync, bit3:use list(XmlHttpSyncUrlPatterns) for disable sync
int s_ini_GzipContentEncoding = 7;	// bit1:my inflate proc enable, bit2:accept gzip enable, bit3:wininet inflate enable
char s_modulePath[MAX_PATH];
int s_ini_HttpSpecialSetting = 0;	
// bit1:remove If-Modified heder, bit2: disable Plugin Windowless-mode, bit3 : window.focus() security check, bit4 : wmode-transparent
// top8bit(0x??000000) : WinInet max Connection Count
#define URLPATTERNS_SIZE 4096
char s_XmlHttpSyncUrlPatternsGET[URLPATTERNS_SIZE];
char s_XmlHttpSyncUrlPatternsPOST[URLPATTERNS_SIZE];

int isXmlHttpSyncUrlGET(const char* szUrl) {
	char* pos = s_XmlHttpSyncUrlPatternsGET;
	while (*pos) {
		if (strstr(szUrl, pos) != NULL) return 1;
		pos += (strlen(pos) + 1);
	}
	return 0;
}

int isXmlHttpSyncUrlPOST(const char* szUrl) {
	char* pos = s_XmlHttpSyncUrlPatternsPOST;
	while (*pos) {
		if (strstr(szUrl, pos) != NULL) return 1;
		pos += (strlen(pos) + 1);
	}
	return 0;
}

char* getModuleFolderName() {
	return s_modulePath;
}

int isProcessImmersive() {
	return s_IsProcessImmersive;
}

int httpSpecialSetting() {
	return s_ini_HttpSpecialSetting;
}

int gzipContentEncoding() {
	return s_ini_GzipContentEncoding;
}

int httpSynchronousMode() {
	return s_ini_HttpSynchronousMode;
}

int enableDebugPrint() {
	return s_ini_EnableDebugPrint;
}
int memCheckInterval() {
	return s_ini_MemCheckInterval;
}
int maxMemoryLoad() {
	return s_ini_MaxMemoryLoad;
}
DWORD maxWorkingSet() {
	return s_ini_MaxWorkingSet;
}

DWORD httpUseProtocolCachePolicy() {
	return s_ini_HttpUseProtocolCachePolicy;
}
DWORD httpReloadIgnoringCacheData() {
	return s_ini_HttpReloadIgnoringCacheData;
}
DWORD httpReturnCacheDataElseLoad() {
	return s_ini_HttpReturnCacheDataElseLoad;
}
DWORD httpReturnCacheDataDontLoad() {
	return s_ini_HttpReturnCacheDataDontLoad;
}
int httpDefaultTimeOut() {
	return s_ini_HttpDefaultTimeOut;
}

int gesturePanEnable() {
	return s_ini_GesturePanEnable;
}
int transform3d() {
	return s_ini_TransForm3D;
}

static bool IsProcessImmersive(HANDLE process) {
	typedef BOOL (WINAPI* IsImmersiveProcessFunc)(HANDLE process);
	HMODULE user32 = ::GetModuleHandleA("user32.dll");
	if (user32 != NULL) {
		IsImmersiveProcessFunc is_immersive_process = reinterpret_cast<IsImmersiveProcessFunc>(::GetProcAddress(user32, "IsImmersiveProcess"));
		if (is_immersive_process) {
			if (is_immersive_process(process)) {
				return true;
			}
		}
	}
	return false;
}

void initializeIniValue() {
	::GetModuleFileNameA(s_instanceHandle, s_modulePath, MAX_PATH);
	char* p = strrchr(s_modulePath, '\\');
	*(p + 1) = 0;
	char dir[MAX_PATH];
	strcpy(dir, s_modulePath);
	strcat(dir, "WebKit.ini");
	s_ini_EnableDebugPrint = GetPrivateProfileIntA("SpecialSettings", "EnableDebugPrint", 0, dir);
	s_ini_TransForm3D = GetPrivateProfileIntA("SpecialSettings", "transform3d", 0, dir);

	s_ini_MemCheckInterval = GetPrivateProfileIntA("garbageCollect", "MemCheckInterval", 40, dir);
	s_ini_MaxMemoryLoad = GetPrivateProfileIntA("garbageCollect", "MaxMemoryLoad", 60, dir);
	s_ini_MaxWorkingSet = (DWORD)GetPrivateProfileIntA("garbageCollect", "MaxWorkingSet", 1610612736, dir);

	s_ini_HttpUseProtocolCachePolicy = (DWORD)GetPrivateProfileIntA("http", "UseProtocolCachePolicy", 0x8440c000, dir);
	s_ini_HttpReloadIgnoringCacheData = (DWORD)GetPrivateProfileIntA("http", "ReloadIgnoringCacheData", 0x8440c000, dir);
	s_ini_HttpReturnCacheDataElseLoad = (DWORD)GetPrivateProfileIntA("http", "ReturnCacheDataElseLoad", 0x8440c000, dir);
	s_ini_HttpReturnCacheDataDontLoad = (DWORD)GetPrivateProfileIntA("http", "ReturnCacheDataDontLoad", 0x8440c000, dir);
	s_ini_HttpDefaultTimeOut = GetPrivateProfileIntA("http", "DefaultTimeOut", 0, dir);
	s_ini_HttpSynchronousMode = GetPrivateProfileIntA("http", "SynchronousMode", 5, dir);
	s_ini_GzipContentEncoding = GetPrivateProfileIntA("http", "gzipContentEncoding", 7, dir);
	s_ini_HttpSpecialSetting = GetPrivateProfileIntA("http", "httpSpecialSetting", 0, dir);

	char seps[] = " ,\t\n";
	char *token;
	ZeroMemory(s_XmlHttpSyncUrlPatternsGET, URLPATTERNS_SIZE);
	if (GetPrivateProfileStringA("http", "XmlHttpSyncUrlPatternsGET", "", s_XmlHttpSyncUrlPatternsGET, URLPATTERNS_SIZE, dir) > 0) {
		token = strtok(s_XmlHttpSyncUrlPatternsGET, seps);
		while (token != NULL) {
		      token = strtok(NULL, seps);
		}
	}
	ZeroMemory(s_XmlHttpSyncUrlPatternsPOST, URLPATTERNS_SIZE);
	if (GetPrivateProfileStringA("http", "XmlHttpSyncUrlPatternsPOST", "", s_XmlHttpSyncUrlPatternsPOST, URLPATTERNS_SIZE, dir) > 0) {
		token = strtok(s_XmlHttpSyncUrlPatternsPOST, seps);
		while (token != NULL) {
		      token = strtok(NULL, seps);
		}
	}

	s_ini_GesturePanEnable = GetPrivateProfileIntA("Gesture", "PanEnable", 2, dir);
	if (GetFileAttributesA("C:\\Windows\\DebugWebKit.txt") != 0xFFFFFFFF) {
		s_ini_EnableDebugPrint = 2;
	}
	s_IsProcessImmersive = (int)(IsProcessImmersive(::GetCurrentProcess()));
}

void printDebugMessage(char* funcname, int line, char* str, ...) {
	if (s_ini_EnableDebugPrint < 0) return;
	va_list arg;
	va_start(arg, str);
	int iTmpEnableDebugPrint = s_ini_EnableDebugPrint;
	s_ini_EnableDebugPrint = -2;
	char cbuf[1024];
	char fmt[256];
	char *mbuf = cbuf;
	int iLen = sprintf_s(fmt, 255, "%s(%d):%s", funcname, line, str);
	if (iLen < 0) iLen = 0;
	fmt[iLen] = 0;
	iLen = _vscprintf( fmt, arg ) + 1;	// add for null terminate
	if (iLen > 1024) {
		mbuf = (char*)malloc(iLen * sizeof(char));
	} 
	int iLen2 = vsprintf_s(mbuf, iLen, fmt, arg);	// iLen𒴂ꍇAPG[𔭐Ă邾I
	if (iLen2 < 0) iLen2 = 0;
	mbuf[iLen2] = 0;
	OutputDebugStringA(mbuf);
	if (iLen > 1024) {
		free(mbuf);
	}
	s_ini_EnableDebugPrint = iTmpEnableDebugPrint;
	va_end(arg);
}

void printDebugError(char* funcname, int line, char* str, DWORD errcode) {
	LPVOID string;
	FormatMessageA(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, errcode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),(LPSTR)&string,0,NULL);
	if (string) {
		printDebugMessage(funcname, line, "Error !!! %s : %d : %s\n", str, errcode, string);
	} else {
		printDebugMessage(funcname, line, "Error !!! %s : %d : (unknown)\n", str, errcode);
	}
	LocalFree(string);
}

} // namespace WebCore
