/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WebCoreInstanceHandle_h
#define WebCoreInstanceHandle_h

#define MYDEBUG 1
#ifdef MYDEBUG
#define MYDEBUGMSG( str, ... )  {\
	if (WebCore::enableDebugPrint() == -2) \
		OutputDebugStringA("#### Another Thread is Blokking DebugPrint !! ####\n"); \
	else \
		if (WebCore::enableDebugPrint() == 2) WebCore::printDebugMessage(__FUNCTION__, __LINE__, str, __VA_ARGS__); }

#define MYDEBUGERR( str, errcode )  {\
	if (WebCore::enableDebugPrint() == 2) WebCore::printDebugError(__FUNCTION__, __LINE__, str, errcode); }

#else
#define MYDEBUGMSG( str, ... )
#define MYDEBUGERR( str, errcode )
#endif

#define JSDEBUGMSG( str, ... )  {\
	if (WebCore::enableDebugPrint() == -2) \
		OutputDebugStringA("#### Another Thread is Blokking DebugPrint !! ####\n"); \
	else \
		if (WebCore::enableDebugPrint() != 0) WebCore::printDebugMessage(__FUNCTION__, __LINE__, str, __VA_ARGS__); }

typedef struct HINSTANCE__* HINSTANCE;

namespace WebCore {

// The global DLL or application instance used for all windows.
extern HINSTANCE s_instanceHandle;

void printDebugMessage(char* funcname, int line, char* str, ...);
void printDebugError(char* funcname, int line, char* str, DWORD errcode);
void initializeIniValue();

inline void setInstanceHandle(HINSTANCE instanceHandle) { s_instanceHandle = instanceHandle; initializeIniValue(); }
inline HINSTANCE instanceHandle() { return s_instanceHandle; }
    
int enableDebugPrint();
int memCheckInterval();
int maxMemoryLoad();
DWORD maxWorkingSet();
DWORD httpUseProtocolCachePolicy();
DWORD httpReloadIgnoringCacheData();
DWORD httpReturnCacheDataElseLoad();
DWORD httpReturnCacheDataDontLoad();
int httpDefaultTimeOut();
int gesturePanEnable();
int transform3d();
int httpSynchronousMode();	// 0:WebKit.org original, 1:use webkitinet.exe., 2:always ASync on XMLHttpRequest, 3:both
int isProcessImmersive();
int gzipContentEncoding();
int httpSpecialSetting();
char* getModuleFolderName();
int isXmlHttpSyncUrlGET(const char* szUrl);
int isXmlHttpSyncUrlPOST(const char* szUrl);

}

#endif // WebCoreInstanceHandle_h
