#include "WebKitPrefix.h"
#include "TSFEventRouter.h"
#include "TSFTextStore.h"

using namespace WebCore;

TSFEventRouter::TSFEventRouter(void)
:m_hWnd(NULL)
,m_frame(NULL)
,m_bInitiated(FALSE)
{
	

}

TSFEventRouter::~TSFEventRouter(void)
{
	Finalize();
}

void TSFEventRouter::SetFrame(RefPtr<WebCore::Frame> frame, HWND hWnd)
{
	HWND_TSF_INFO* pInfo = FindInfo(hWnd);
	if (pInfo) {
		if(pInfo->pTextStore){
			pInfo->pTextStore->SetFrame(frame);
		}
	}
}

BOOL TSFEventRouter::SelectionChange(HWND hWnd) {

	HWND_TSF_INFO* pInfo = FindInfo(hWnd);
	if (pInfo) {
		if(pInfo->pTextStore) {

			pInfo->pTextStore->OnSelectionChange();
		}
	}
	return TRUE;
}

VOID TSFEventRouter::Initialize()
{
	if (m_bInitiated) {
		return;
	}

	CoInitialize(0);
	if(FAILED(CoCreateInstance(CLSID_TF_ThreadMgr,NULL,CLSCTX_INPROC_SERVER, 
		IID_ITfThreadMgr,(void**)&m_pThreadMgr)))
	{

		return ;
	}

	if(FAILED(m_pThreadMgr->Activate(&m_tfClientId)))
	{

		return ;
	}

	if(m_pThreadMgr->QueryInterface(IID_ITfKeystrokeMgr,(void**)&m_pKeystoreMgr) != S_OK)
	{
		return ;
	}

	m_bInitiated = TRUE;
}

VOID TSFEventRouter::Finalize()
{
	if (!m_bInitiated) {
		return;
	}
	m_bInitiated = FALSE;
	for (int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfo = m_HwndTSFInfoVec.at(i);
		if (pInfo) {
			if (pInfo->pContext && pInfo->dwTextEditSinkCookie) {
				ITfSource* pSource = NULL;
				pInfo->pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
				pSource->UnadviseSink(pInfo->dwTextEditSinkCookie);
			}

			if (pInfo->pDocumentMgr) {
				pInfo->pDocumentMgr->Pop(TF_POPF_ALL);
			}
			if (pInfo->pContext) {
				pInfo->pContext->Release();
				pInfo->pContext = NULL;
			}
			if (pInfo->pPrevDocmentMgr) {
				m_pThreadMgr->AssociateFocus(pInfo->hWnd, pInfo->pPrevDocmentMgr, NULL);
			}
			if (pInfo->pDocumentMgr) {
				pInfo->pDocumentMgr->Release();
				pInfo->pDocumentMgr = NULL;
			}
			if (pInfo->pTextStore) {
				pInfo->pTextStore->Release();
			}
			delete pInfo;
		}
	}
	m_HwndTSFInfoVec.clear();
	if (m_pThreadMgr) {
		m_pThreadMgr->Deactivate();
		m_pThreadMgr->Release();
		m_pThreadMgr = NULL;
	}
	CoUninitialize();
}

VOID TSFEventRouter::AddWindow( HWND hWnd )
{
	if (!m_pThreadMgr) {
		return;
	}
	ITfDocumentMgr *pDocumentMgr = NULL;
	m_pThreadMgr->CreateDocumentMgr(&pDocumentMgr);
	TSFTextStore* pTSFTextStore = new TSFTextStore();
	pTSFTextStore->m_hWnd = hWnd;
	TfEditCookie ecTextStore;
	ITfContext* pContext = NULL;
	if (!pDocumentMgr) {
		return;
	}
	pDocumentMgr->CreateContext(m_tfClientId, 0, (ITextStoreACP*)pTSFTextStore, &pContext, &ecTextStore);
	pTSFTextStore->m_EditCookie = ecTextStore;
	pDocumentMgr->Push(pContext);
	ITfDocumentMgr* pPrevDocumentMgr = NULL;
	//m_pThreadMgr->AssociateFocus(hWnd, pDocumentMgr, &pPrevDocumentMgr);
	ITfSource *pSource = NULL;
	if (!pContext) {
		return;
	}
	pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
	DWORD dwTextEditSinkCookie = 0;
	if (!pSource) {
		return;
	}
	pSource->AdviseSink(IID_ITfTextEditSink, (ITfTextEditSink*)pTSFTextStore, &dwTextEditSinkCookie);

	HWND_TSF_INFO *pInfo = new HWND_TSF_INFO();
	pInfo->hWnd = hWnd;
	pInfo->pDocumentMgr = pDocumentMgr;
	pInfo->pContext = pContext;
	pInfo->pTextStore = pTSFTextStore;
	pInfo->pPrevDocmentMgr = pPrevDocumentMgr;
	pInfo->dwTextEditSinkCookie = dwTextEditSinkCookie;

	m_HwndTSFInfoVec.append(pInfo);

}

INT TSFEventRouter::DeleteWindow( HWND hWnd )
{
	for(int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfo = m_HwndTSFInfoVec.at(i);
		if (pInfo) {
			if (pInfo->hWnd == hWnd) {
				if (pInfo->pContext && pInfo->dwTextEditSinkCookie) {
					ITfSource* pSource = NULL;
					pInfo->pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
					pSource->UnadviseSink(pInfo->dwTextEditSinkCookie);
				}

				if (pInfo->pDocumentMgr) {
					pInfo->pDocumentMgr->Pop(TF_POPF_ALL);
				}
				if (pInfo->pContext) {
					pInfo->pContext->Release();
					pInfo->pContext = NULL;
				}
				if (pInfo->pPrevDocmentMgr) {
					m_pThreadMgr->AssociateFocus(pInfo->hWnd, pInfo->pPrevDocmentMgr, NULL);
				}
				if (pInfo->pDocumentMgr) {
					pInfo->pDocumentMgr->Release();
					pInfo->pDocumentMgr = NULL;
				}
				if (pInfo->pTextStore) {
					pInfo->pTextStore->Release();
				}
				delete pInfo;
				m_HwndTSFInfoVec.remove(i);

				break;
			}
		}
	}
	if (!m_HwndTSFInfoVec.size()) {
		Finalize();
	}
	return m_HwndTSFInfoVec.size();
}

TSFEventRouter::HWND_TSF_INFO* TSFEventRouter::FindInfo( HWND hWnd )
{
	HWND_TSF_INFO* pInfo = NULL;
	for (int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfoTemp = m_HwndTSFInfoVec.at(i);
		if (pInfoTemp) {
			if (pInfoTemp->hWnd == hWnd) {
				pInfo = pInfoTemp;
				break;
			}
		}
	}
	return pInfo;
}

