#ifndef TSFTextStore_h
#define TSFTextStore_h

#include "config.h"
#include "textstor.h"
#include "Range.h"
#include "WebCore/Frame.h"
#include <msctf.h>


using namespace WebCore;


class TSFTextStore :
	public ITfContextOwnerCompositionSink,
	public ITextStoreACP,
	public ITfTextEditSink
{
public:
	TSFTextStore(void);
	virtual ~TSFTextStore(void);

	void OnSelectionChange();
	RefPtr<Range> GetRange(LONG &,LONG &);	
	//
    // IUnknown methods
    //
    STDMETHODIMP QueryInterface(REFIID riid, void **ppvObj);
    STDMETHODIMP_(ULONG) AddRef(void);
    STDMETHODIMP_(ULONG) Release(void);

    //
    // ITextStoreACP
    //
    STDMETHODIMP    AdviseSink(REFIID riid, IUnknown *punk, DWORD dwMask);
    STDMETHODIMP    UnadviseSink(IUnknown *punk);
    STDMETHODIMP    RequestLock(DWORD dwLockFlags, HRESULT *phrSession);
    STDMETHODIMP    GetStatus(TS_STATUS *pdcs);
    STDMETHODIMP    QueryInsert(LONG acpInsertStart, LONG acpInsertEnd, ULONG cch, LONG *pacpResultStart, LONG *pacpResultEnd);
    STDMETHODIMP    GetSelection(ULONG ulIndex, ULONG ulCount, TS_SELECTION_ACP *pSelection, ULONG *pcFetched);
    STDMETHODIMP    SetSelection(ULONG ulCount, const TS_SELECTION_ACP *pSelection);
    STDMETHODIMP    GetText(LONG acpStart, LONG acpEnd, __out_ecount(cchPlainReq) WCHAR *pchPlain, ULONG cchPlainReq, ULONG *pcchPlainOut, TS_RUNINFO *prgRunInfo,
        ULONG ulRunInfoReq, ULONG *pulRunInfoOut, LONG *pacpNext);
    STDMETHODIMP    SetText(DWORD dwFlags, LONG acpStart, LONG acpEnd, __in_ecount(cch) const WCHAR *pchText, ULONG cch, TS_TEXTCHANGE *pChange);
    STDMETHODIMP    GetFormattedText(LONG acpStart, LONG acpEnd, IDataObject **ppDataObject);
    STDMETHODIMP    GetEmbedded(LONG acpPos, REFGUID rguidService, REFIID riid, IUnknown **ppunk);
    STDMETHODIMP    InsertEmbedded(DWORD dwFlags, LONG acpStart, LONG acpEnd, IDataObject *pDataObject, TS_TEXTCHANGE *pChange);
    STDMETHODIMP    RequestSupportedAttrs(DWORD dwFlags, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs);
    STDMETHODIMP    RequestAttrsAtPosition(LONG acpPos, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs, DWORD dwFlags);
    STDMETHODIMP    RequestAttrsTransitioningAtPosition(LONG acpPos, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs, DWORD dwFlags);
    STDMETHODIMP    FindNextAttrTransition(LONG acpStart, LONG acpHalt, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs,
        DWORD dwFlags, LONG *pacpNext, BOOL *pfFound, LONG *plFoundOffset);
    STDMETHODIMP    RetrieveRequestedAttrs(ULONG ulCount, TS_ATTRVAL *paAttrVals, ULONG *pcFetched);
    STDMETHODIMP    GetEndACP(LONG *pacp);
    STDMETHODIMP    GetActiveView(TsViewCookie *pvcView);
    STDMETHODIMP    GetACPFromPoint(TsViewCookie vcView, const POINT *pt, DWORD dwFlags, LONG *pacp);
    STDMETHODIMP    GetTextExt(TsViewCookie vcView, LONG acpStart, LONG acpEnd, RECT *prc, BOOL *pfClipped);
    STDMETHODIMP    GetScreenExt(TsViewCookie vcView, RECT *prc);
    STDMETHODIMP    GetWnd(TsViewCookie vcView, HWND *phwnd);
    STDMETHODIMP    QueryInsertEmbedded(const GUID *pguidService, const FORMATETC *pFormatEtc, BOOL *pfInsertable);
    STDMETHODIMP    InsertTextAtSelection(DWORD dwFlags, __in_ecount(cch) const WCHAR *pchText, ULONG cch, LONG *pacpStart,
        LONG *pacpEnd, TS_TEXTCHANGE *pChange);
    STDMETHODIMP    InsertEmbeddedAtSelection(DWORD dwFlags, IDataObject *pDataObject, LONG *pacpStart, 
        LONG *pacpEnd, TS_TEXTCHANGE *pChange);

	STDMETHODIMP  OnEndEdit(ITfContext* context,TfEditCookie read_only_edit_cookie,ITfEditRecord* edit_record) OVERRIDE; 

	ITextStoreACPSink * m_pSink;

	WTF::String m_stringBuffer;


	void SetFrame(RefPtr<WebCore::Frame> frame);
	//RefPtr<WebCore::Range> m_range;

	RefPtr<WebCore::Frame> m_frame;
	HWND m_hWnd;

    long _cRef;


    //
    // ITfContextOwnerCompositionSink
    //
    STDMETHODIMP OnStartComposition(ITfCompositionView *pComposition,
                                    BOOL *pfOk);

    STDMETHODIMP OnUpdateComposition(ITfCompositionView *pComposition,
                                     ITfRange *pRangeNew);

    STDMETHODIMP OnEndComposition(ITfCompositionView *pComposition);

    ITfCompositionView *_pCurrentCompositionView;
	Vector<CompositionUnderline> lines;


	TfEditCookie m_EditCookie;

	Vector<DWORD> m_LockQueue;
	DWORD m_dwCurLock;

	ULONG m_uSelStart;
	ULONG m_uSelEnd;
	BOOL GetCompostionStatus(ITfContext* context, TfEditCookie read_only_edit_cookie);
	ITfCategoryMgr *m_pcat;
	ITfDisplayAttributeMgr *m_pdam;

};
#endif;
