/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include <WebOpenPanelResultListener.h>
#include <wtf/text/CString.h>

static inline String toString(BSTR bstr)
{
    return String(bstr, SysStringLen(bstr));
}

WebOpenPanelResultListener*  WebOpenPanelResultListener::createInstance(PassRefPtr<WebCore::FileChooser> fileChooser)
{
	WebOpenPanelResultListener * instance =  new WebOpenPanelResultListener(fileChooser) ;
	instance->AddRef();
	return instance;

}

WebOpenPanelResultListener::WebOpenPanelResultListener(PassRefPtr<WebCore::FileChooser> fileChooser)
:m_refCount(0),m_fileChooser(fileChooser)
{
}

WebOpenPanelResultListener::~WebOpenPanelResultListener()
{
	//Release();
}

HRESULT STDMETHODCALLTYPE WebOpenPanelResultListener::chooseFilesnames(/*[in]*/ IEnumVARIANT** fileName)
{
	return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE WebOpenPanelResultListener::chooseFilename(/* [in] */ BSTR fileName) {
	if(!m_fileChooser) {
		return S_FALSE;
	}

	Vector<String> filenames;
	WCHAR* temp = (WCHAR*)fileName;
	String name(temp);
	filenames.append(name);
    m_fileChooser.get()->chooseFiles(filenames);
	return S_OK;
		
}

HRESULT STDMETHODCALLTYPE WebOpenPanelResultListener::cancel( void) {
	if(!m_fileChooser) {
		return S_FALSE;
	}

	m_fileChooser = NULL;

	return S_OK;
}

// IUnknown -------------------------------------------------------------------
HRESULT STDMETHODCALLTYPE WebOpenPanelResultListener::QueryInterface(REFIID riid, void** ppvObject)
{
    *ppvObject = 0;
    if (IsEqualGUID(riid, IID_IUnknown))
		*ppvObject = static_cast<IWebOpenPanelResultListener*>(this);
    else if (IsEqualGUID(riid, IID_IWebOpenPanelResultListener))
        *ppvObject = static_cast<IWebOpenPanelResultListener*>(this);
    else
        return E_NOINTERFACE;

    AddRef();
    return S_OK;
}

ULONG STDMETHODCALLTYPE WebOpenPanelResultListener::AddRef(void)
{
    return ++m_refCount;
}

ULONG STDMETHODCALLTYPE WebOpenPanelResultListener::Release(void)
{
    ULONG newRef = --m_refCount;
    if (!newRef)
        delete(this);

    return newRef;
}


//} // namespace WebKit
